import { HttpService } from '@nestjs/axios';
import { CACHE_MANAGER } from '@nestjs/cache-manager';
import {
  BadRequestException,
  ConflictException,
  Inject,
  Injectable,
  NotFoundException,
  UnauthorizedException,
} from '@nestjs/common';
import { JwtService } from '@nestjs/jwt';
import { InjectRepository } from '@nestjs/typeorm';
import * as bcrypt from 'bcrypt';
import { Cache } from 'cache-manager';
import { Account } from 'src/account/entities/account.entity';
import { DefaultStatus, LogType, LoginType, UserRole } from 'src/enum';
import { UserDetail } from 'src/user-details/entities/user-detail.entity';
import { UserPermission } from 'src/user-permissions/entities/user-permission.entity';
import APIFeatures from 'src/utils/apiFeatures.utils';
import { Repository } from 'typeorm';
import {
  ForgotPassDto,
  OtpDto,
  SigninDto,
  UserRegisterDto,
  VerifyOtpDto,
} from './dto/login.dto';
import { NodeMailerService } from 'src/node-mailer/node-mailer.service';

@Injectable()
export class AuthService {
  constructor(
    private jwtService: JwtService,
    @InjectRepository(Account) private readonly repo: Repository<Account>,
    @InjectRepository(UserPermission)
    private readonly upRepo: Repository<UserPermission>,
    @InjectRepository(UserDetail)
    private readonly userDetailRepo: Repository<UserDetail>,
    private readonly httpService: HttpService,
    @Inject(CACHE_MANAGER) private cacheManager: Cache,
    private readonly nodeMailerService: NodeMailerService,
  ) {}

  async signIn(loginId: string, password: string) {
    const admin = await this.getUserDetails(loginId, UserRole.ADMIN);
    const comparePassword = await bcrypt.compare(password, admin.password);
    if (!comparePassword) {
      throw new UnauthorizedException('Invalid Credentials');
    }
    const token = await APIFeatures.assignJwtToken(admin.id, this.jwtService);
    return { token };
  }

  async userLogin(dto: SigninDto) {
    const user = await this.repo.findOne({
      where: { phoneNumber: dto.phoneNumber, roles: UserRole.USER },
    });
    if (!user) {
      throw new NotFoundException('Account not found, Please register first!');
    }
    const otp = 783200;
    // const otp = Math.floor(1000 + Math.random() * 9000);
    // sendOtp(+dto.phoneNumber, otp);
    this.cacheManager.set(dto.phoneNumber, otp, 10 * 60 * 1000);

    return { phoneNumber: dto.phoneNumber, message: 'OTP sent succesfully' };
  }

  async verifyOtp(dto: OtpDto) {
    const user = await this.getUserDetails(dto.phoneNumber, UserRole.USER);
    if (!user) {
      throw new NotFoundException('User not found with this Phone Number!');
    }
    const sentOtp = await this.cacheManager.get(dto.phoneNumber);
    if (!sentOtp) {
      throw new UnauthorizedException('OTP expired or not found!');
    }
    if (dto.otp !== sentOtp) {
      throw new UnauthorizedException('Invalid OTP!');
    }
    const token = await APIFeatures.assignJwtToken(user.id, this.jwtService);
    await this.cacheManager.del(dto.phoneNumber);
    return { token, accountId: user.id };
  }

  async sentOtp(dto: SigninDto) {
    const user = await this.repo.findOne({
      where: { phoneNumber: dto.phoneNumber, roles: UserRole.USER },
    });
    if (user) {
      throw new ConflictException(
        'Phone Number already exists with another account!',
      );
    }
    const otp = 783200;
    // const otp = Math.floor(1000 + Math.random() * 9000);
    // sendOtp(+dto.phoneNumber, otp);
    this.cacheManager.set(dto.phoneNumber, otp, 10 * 60 * 1000);

    return {
      phoneNumber: dto.phoneNumber,
      success: true,
      message: 'OTP sent succesfully',
    };
  }

  async verifyPhone(dto: OtpDto) {
    const storedOtp = await this.cacheManager.get(dto.phoneNumber);
    if (!storedOtp) {
      throw new UnauthorizedException('OTP expired or not found!');
    }
    if (storedOtp !== dto.otp) {
      throw new BadRequestException('Invalid or expired OTP');
    }
    return {
      phoneNumber: dto.phoneNumber,
      success: true,
      message: 'OTP Macthed.',
    };
  }

  async userRegister(dto: UserRegisterDto) {
    const checkUser = await this.repo
      .createQueryBuilder('account')
      .where('account.phoneNumber = :phoneNumber AND account.roles = :roles', {
        phoneNumber: dto.phoneNumber,
        roles: UserRole.USER,
      })
      .getOne();
    if (checkUser) {
      throw new ConflictException(
        'Phone Number already exists with another account!',
      );
    }

    const userId = this.generateUserId();
    const account = await this.repo.create({
      phoneNumber: dto.phoneNumber,
      roles: UserRole.USER,
      status: DefaultStatus.ACTIVE,
      type: LoginType.PHONE,
    });
    await this.repo.save(account);

    const udObj = await this.userDetailRepo.create({
      accountId: account.id,
      name: dto.name,
      email: dto.email,
      userId: userId,
    });
    await this.userDetailRepo.save(udObj);

    return account;
  }

  private generateUserId(): string {
    const letters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'.split('');
    for (let i = letters.length - 1; i > 0; i--) {
      const j = Math.floor(Math.random() * (i + 1));
      [letters[i], letters[j]] = [letters[j], letters[i]];
    }
    const randLetter = letters.slice(0, 4).join('');
    return `${randLetter}${Math.floor(1000000 + Math.random() * 9000000)}`;
  }

  validate(id: string) {
    return this.getUserDetails(id);
  }

  findPermission(accountId: string) {
    return this.getPermissions(accountId);
  }

  private getPermissions = async (accountId: string): Promise<any> => {
    let result = await this.cacheManager.get('userPermission' + accountId);
    if (!result) {
      result = await this.upRepo.find({
        relations: ['permission', 'menu'],
        where: { accountId, status: true },
      });
      this.cacheManager.set(
        'userPermission' + accountId,
        result,
        7 * 24 * 60 * 60 * 1000,
      );
    }
    return result;
  };

  private getUserDetails = async (
    id: string,
    role?: UserRole,
  ): Promise<any> => {
    const query = this.repo
      .createQueryBuilder('account')
      .leftJoinAndSelect('account.userDetail', 'userDetail');
    if (!role && role == UserRole.USER) {
      query.where('account.roles = :roles', { roles: UserRole.USER });
    }
    if (!role && role == UserRole.ADMIN) {
      query.where('account.roles IN (:...roles)', {
        roles: [UserRole.ADMIN, UserRole.STAFF],
      });
    }
    const result = await query
      .andWhere('account.id = :id OR account.phoneNumber = :phoneNumber', {
        id: id,
        phoneNumber: id,
      })
      .getOne();
    if (!result) {
      throw new UnauthorizedException('Account not found!');
    }
    return result;
  };
}
