import {
  BadRequestException,
  ConflictException,
  Injectable,
  NotAcceptableException,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { DefaultStatus } from 'src/enum';
import { Brackets, Repository } from 'typeorm';
import { CategoryDto, StatusDto } from './dto/category.dto';
import { Category } from './entities/category.entity';
import { Poster } from 'src/poster/entities/poster.entity';

@Injectable()
export class CategoryService {
  constructor(
    @InjectRepository(Category) private readonly repo: Repository<Category>,
    @InjectRepository(Poster) private readonly posterRepo: Repository<Poster>,
  ) {}

  async create(dto: CategoryDto) {
    const category = await this.repo.findOne({ where: { name: dto.name } });
    if (category) {
      throw new ConflictException('Category already exists!');
    }
    const obj = Object.assign(dto);
    return this.repo.save(obj);
  }

  async increaseCount(categoryId: string, posterId: string) {
    const category = await this.repo.findOne({ where: { id: categoryId } });
    const poster = await this.posterRepo.findOne({ where: { id: posterId } });

    if (!category) {
      throw new NotFoundException('Category Not Found!');
    }

    if (!poster) {
      throw new NotFoundException('Poster Not Found!');
    }

    category.downloadCount++;
    await this.repo.save(category);

    poster.downloadCount++;
    await this.posterRepo.save(poster);

    return { countIncrese: true };
  }

  async findAll(
    limit: number,
    offset: number,
    keyword: string,
    status: DefaultStatus,
  ) {
    const [result, total] = await this.repo
      .createQueryBuilder('category')
      .where('category.status = :status', { status: status })
      .andWhere(
        new Brackets((qb) => {
          qb.where('category.name LIKE :pname', {
            pname: '%' + keyword + '%',
          });
        }),
      )
      .orderBy(
        `CASE WHEN category.name LIKE '${keyword}%' THEN 0 ELSE 1 END, category.name`,
        'ASC',
      )
      .take(limit)
      .skip(offset)
      .getManyAndCount();

    return { result, total };
  }

  async find() {
    const [result, total] = await this.repo
      .createQueryBuilder('category')
      .leftJoinAndSelect(
        'category.poster',
        'poster',
        'poster.status = :status',
        { status: DefaultStatus.ACTIVE },
      )
      .leftJoinAndSelect('poster.specialDay', 'specialDay')
      .select([
        'category.id',
        'category.name',
        'category.downloadCount',
        'category.status',
        'category.createdAt',

        'poster.id',
        'poster.poster',
        'poster.date',
        'poster.type',
        'poster.downloadCount',
        'poster.status',
        'poster.createdAt',

        'specialDay.id',
        'specialDay.name',
        'specialDay.month',
        'specialDay.day',
        'specialDay.status',
      ])
      .where('category.status = :status', { status: DefaultStatus.ACTIVE })
      .getManyAndCount();
    return { result, total };
  }

  async findOne(id: string) {
    const category = await this.repo.findOne({ where: { id: id } });
    if (!category) {
      throw new NotFoundException('Category not found!');
    }
    return category;
  }

  async update(id: string, dto: CategoryDto) {
    try {
      const category = await this.repo.findOne({ where: { id } });
      if (!category) {
        throw new NotFoundException('Category not found!');
      }
      const obj = Object.assign(category, dto);
      return this.repo.save(obj);
    } catch (error) {
      throw new NotAcceptableException(
        'Either catgeory exists or invalid name!',
      );
    }
  }

  async status(id: string, dto: StatusDto) {
    const catgeory = await this.repo.findOne({ where: { id: id } });
    if (!catgeory) {
      throw new NotFoundException('Category not found!');
    }
    const obj = Object.assign(catgeory, dto);
    return this.repo.save(obj);
  }
}
