import { Injectable, NotFoundException } from '@nestjs/common';
import {
  CreatePosterDto,
  PosterPaginationDto,
  PosterTypeDto,
  SearchPosterPaginationDto,
  UserPosterPaginationDto,
} from './dto/create-poster.dto';
import { UpdatePosterDto } from './dto/update-poster.dto';
import { Poster } from './entities/poster.entity';
import { InjectRepository } from '@nestjs/typeorm';
import { Brackets, Repository } from 'typeorm';
import { DefaultStatusDto } from 'src/common/dto/default-status.dto';
import { DefaultStatus } from 'src/enum';

@Injectable()
export class PosterService {
  constructor(
    @InjectRepository(Poster)
    private readonly repo: Repository<Poster>,
  ) {}

  async create(image: string, categoryId: string, specialDayId: string) {
    const obj = Object.assign({
      poster: process.env.POLY_UPLOAD_LINK + image,
      posterPath: image,
      categoryId: categoryId,
      specialDayId: specialDayId,
    });
    return this.repo.save(obj);
  }

  async createByCategory(image: string, categoryId: string) {
    const obj = Object.assign({
      poster: process.env.POLY_UPLOAD_LINK + image,
      posterPath: image,
      categoryId: categoryId,
    });
    return this.repo.save(obj);
  }

  async findAll(dto: PosterPaginationDto) {
    const keyword = dto.keyword || '';
    const query = await this.repo
      .createQueryBuilder('poster')
      .leftJoinAndSelect('poster.category', 'category')
      .leftJoinAndSelect('poster.specialDay', 'specialDay')
      .select([
        'poster.id',
        'poster.categoryId',
        'poster.specialDayId',
        'poster.poster',
        'poster.downloadCount',
        'poster.date',
        'poster.type',
        'poster.status',
        'poster.createdAt',

        'category.id',
        'category.name',
        'category.downloadCount',
        'category.status',

        'specialDay.id',
        'specialDay.name',
        'specialDay.day',
        'specialDay.month',
        'specialDay.status',
        'specialDay.createdAt',
      ])
      .where('poster.status = :status', { status: dto.status });
    if (dto.categoryId && dto.categoryId.length > 0) {
      query.andWhere('poster.categoryId = :categoryId', {
        categoryId: dto.categoryId,
      });
    }
    if (dto.specialDayId && dto.specialDayId.length > 0) {
      query.andWhere('poster.specialDayId = :specialDayId', {
        specialDayId: dto.specialDayId,
      });
    }
    if (dto.type && dto.type.length > 0) {
      query.andWhere('poster.type = :type', {
        type: dto.type,
      });
    }
    if (keyword) {
      query.andWhere(
        new Brackets((qb) => {
          qb.where('poster.date LIKE :keyword', {
            keyword: '%' + keyword + '%',
          });
        }),
      );
    }

    const [result, total] = await query
      .orderBy({ 'poster.createdAt': 'DESC' })
      .take(dto.limit)
      .skip(dto.offset)
      .getManyAndCount();
    return { result, total };
  }

  async findByUser(categoryId: string, dto: UserPosterPaginationDto) {
    const query = await this.repo
      .createQueryBuilder('poster')
      .select([
        'poster.id',
        'poster.categoryId',
        'poster.specialDayId',
        'poster.poster',
        'poster.downloadCount',
        'poster.date',
        'poster.type',
        'poster.status',
        'poster.createdAt',
      ])
      .where('poster.categoryId = :categoryId AND poster.status = :status', {
        status: DefaultStatus.ACTIVE,
        categoryId: categoryId,
      });
    if (dto.specialDayId && dto.specialDayId.length > 0) {
      query.andWhere('poster.specialDayId = :specialDayId', {
        specialDayId: dto.specialDayId,
      });
    }

    const [result, total] = await query
      .take(dto.limit)
      .skip(dto.offset)
      .getManyAndCount();
    return { result, total };
  }

  async searchPoster(dto: SearchPosterPaginationDto) {
    const keyword = dto.keyword || '';
    const query = await this.repo
      .createQueryBuilder('poster')
      .leftJoinAndSelect('poster.category', 'category')
      .select([
        'poster.id',
        'poster.categoryId',
        'poster.specialDayId',
        'poster.poster',
        'poster.downloadCount',
        'poster.date',
        'poster.type',
        'poster.status',
        'poster.createdAt',

        'category.id',
        'category.name',
        'category.downloadCount',
        'category.status',
      ])
      .where('poster.status = :status AND category.status = :cStatus', {
        status: DefaultStatus.ACTIVE,
        cStatus: DefaultStatus.ACTIVE,
      });

    if (dto.categoryId && dto.categoryId.length > 0) {
      query.andWhere('poster.categoryId = :categoryId', {
        categoryId: dto.categoryId,
      });
    }
    if (dto.type && dto.type.length > 0) {
      query.andWhere('poster.type = :type', { type: dto.type });
    }
    if (keyword.length > 0) {
      query.andWhere(
        new Brackets((qb) => {
          qb.where('category.name LIKE :keyword', {
            keyword: '%' + keyword + '%',
          });
        }),
      );
    }

    const [result, total] = await query
      .take(dto.limit)
      .skip(dto.offset)
      .getManyAndCount();
    return { result, total };
  }

  async findOne(id: string) {
    return this.repo.findOne({ where: { id } });
  }

  async dateUpdate(id: string, dto: UpdatePosterDto) {
    const result = await this.repo.findOne({ where: { id: id } });
    if (!result) {
      throw new NotFoundException('Poster Not Found..');
    }
    const parsedDate = new Date(dto.date);

    if (!dto.date || isNaN(parsedDate.getTime())) {
      const obj = Object.assign(result, { date: null });
    return this.repo.save(obj);
    }

    const monthName = parsedDate.toLocaleString('en-US', { month: 'long' });
    const day = parsedDate.getDate();

    const obj = Object.assign(result, { date: `${day} ${monthName}` });
    return this.repo.save(obj);
  }

  async poster(image: string, result: Poster) {
    const obj = Object.assign(result, {
      poster: process.env.POLY_UPLOAD_LINK + image,
      posterPath: image,
    });
    return this.repo.save(obj);
  }

  async status(id: string, dto: DefaultStatusDto) {
    const result = await this.repo.findOne({ where: { id: id } });
    if (!result) {
      throw new NotFoundException('Poster Not Found..');
    }
    const obj = Object.assign(result, { status: dto.status });
    return this.repo.save(obj);
  }

  async type(id: string, dto: PosterTypeDto) {
    const result = await this.repo.findOne({ where: { id: id } });
    if (!result) {
      throw new NotFoundException('Poster Not Found..');
    }
    const obj = Object.assign(result, { type: dto.type });
    return this.repo.save(obj);
  }
}
